package com.altinity.opsview.odw.dao;

/**
 * Copyright (C) 2008 Opsera Limited. All rights reserved
 *
 * This file is part of Opsview
 * 
 * Opsview is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * Opsview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Opsview; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 * 
 */

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

public class ODWDAO {
	
	public static int INTERVAL_DAY = 0;
	public static int INTERVAL_WEEK = 1;
	public static String PERCENT_OK_INDEX = "percentOK";
	public static String PERCENT_SCHED_INDEX = "percentSched";
	public static String PERCENT_UNSCHED_INDEX = "percentUnsched";
	
	protected DataSource dataSource;
	protected SimpleJdbcTemplate jdbcTemplate;	
	
	/**
	 * 
	 */
	public void setDataSource(DataSource dataSource) {
		this.dataSource = dataSource;
	}	
	
	/**
	 * 
	 */
	public DataSource getDataSource() {
		return dataSource;
	}

	/**
	 * 
	 */
	protected SimpleJdbcTemplate getJdbcTemplate() {
		if (jdbcTemplate == null)
			jdbcTemplate = new SimpleJdbcTemplate(dataSource);
	
		return jdbcTemplate;
	}
	
	/**
	 * 
	 */
	public void hitDB() {
		
		StringBuffer query = new StringBuffer();
		query.append("select :text");
		
		Map params = Collections.singletonMap("text", "Testing DB connection");
		
		getJdbcTemplate().query(query.toString(), new SimpleStringMapper(), params);
	}

	/**
	 * 
	 */
	protected void generateInList(StringBuffer query, List <?> list) {
		
		query.append("IN (");
		Iterator i = list.iterator();
		while ( i.hasNext() ) {
			
			query.append("\'");
			query.append(i.next());
			query.append("\'");
			if ( i.hasNext() )
				query.append(", ");
		}
		query.append(")");
	}
	
	/**
	 * 
	 */
	protected class SimpleStringMapper implements ParameterizedRowMapper<String> {

		public String mapRow(ResultSet rs, int rowNum) throws SQLException {
			return rs.getString(1);
		}
	}
	
	/**
	 * 
	 */
	protected class SimpleIntMapper implements ParameterizedRowMapper<Integer> {

		public Integer mapRow(ResultSet rs, int rowNum) throws SQLException {
			return rs.getInt(1);
		}
	}
	
	/**
	 * 
	 */
	protected class SimpleIntToDateMapper implements ParameterizedRowMapper<Date> {

		public Date mapRow(ResultSet rs, int rowNum) throws SQLException {
			return new Date(new Long(rs.getInt(1)) * 1000l);
		}
	}	

}
