package com.altinity.opsview.odw.elements;

/**
 * Copyright (C) 2008 Opsera Limited. All rights reserved
 *
 * This file is part of Opsview
 * 
 * Opsview is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * Opsview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Opsview; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 * 
 */

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.dom4j.Element;

import com.altinity.opsview.odw.ReportConstants;
import com.altinity.opsview.odw.bc.ODWReportBC;
import com.altinity.opsview.odw.bc.ODWReportLogger;
import com.altinity.opsview.odw.dao.ODWReportConfigDAO;
import com.altinity.opsview.odw.dao.ODWReportDAO;
import com.altinity.opsview.odw.exception.ReportException;
import com.altinity.opsview.odw.model.AvailabilityModel;
import com.altinity.opsview.odw.model.HostGroupWeek;
import com.altinity.opsview.odw.model.ReportWeek;
import com.altinity.opsview.odw.reports.ReportCriteria;

public class HostGroupExceptionDetail extends AbstractReportElement implements ReportConstants {
	
	private List<AvailabilityModel> l;

	/**
	 *
	 */
	public HostGroupExceptionDetail(ReportCriteria rc, ODWReportDAO dao, ODWReportConfigDAO configDao, ODWReportLogger log) {
		super(rc, dao, configDao, log);
	}

	/**
	 * 
	 */
	@Override
	public Element addElement(Element elements) {
		
		Element exceptionDetail = elements.addElement("element");
		exceptionDetail.addAttribute("name", "exception-detail");
		exceptionDetail.addElement("title").setText("Exception Detail");
		
		// First handle case where no exceptions exist in reporting period
		if (l.isEmpty()) {
			
			exceptionDetail.addElement("empty").setText("true");
			
			// Do not need to report error as there will be reports with
			// no exceptions
			
			return exceptionDetail;	
		}
		
		// Exceptions found in the reporting period - add them to list
		
		List<ReportWeek> weeks = listReportWeeks();

		Element weeksElement = exceptionDetail.addElement("weeks");
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		for (ReportWeek w : weeks) {
			Element weekElement = weeksElement.addElement("week");
			weekElement.addElement("weekno").setText(w.getWeekOfYear());
			weekElement.addElement("startdate").setText(sdf.format(w.getReportWeekStartDate()));
			weekElement.addElement("enddate").setText(sdf.format(w.getReportWeekEndDate()));
		}

		// first make rows for week
		Map<String, HostGroupWeek> hgWeeks = new LinkedHashMap<String, HostGroupWeek>();
		for (AvailabilityModel aModel : l) {
			String hgName = aModel.getHostGroupName();
			HostGroupWeek hgw = hgWeeks.get(aModel.getHostGroupName());
			if (hgw == null) {
				hgw = new HostGroupWeek(hgName);
				hgWeeks.put(hgName, hgw);
			}
			// now dd the availability
			hgw.addWeek(aModel.getWeek(), aModel);
		}
		// now remove perfecct host groups
		Iterator i = hgWeeks.keySet().iterator();
		while (i.hasNext()) {
			String hostGroup = (String) i.next();
			HostGroupWeek hgw = hgWeeks.get(hostGroup);
			if (hgw.isPerfect())
				i.remove();
		}

		// Now put them in the friggen list
		i = hgWeeks.keySet().iterator();		
		
		Element hostgroups = exceptionDetail.addElement("hostgroups");
		DecimalFormat percentFormat = new DecimalFormat(PERCENT_FORMAT);
		while (i.hasNext()) {
				
			String hostGroup = (String) i.next();
			HostGroupWeek hgw = hgWeeks.get(hostGroup);
			
			Element host = null;
			Element ws = null;
			host = hostgroups.addElement("hostgroup");
			host.addElement("name").setText(hgw.getHostGroup());
			ws = host.addElement("availability");
			for (ReportWeek rw : weeks) {
				
				Element w = ws.addElement("week");
				AvailabilityModel am = hgw.getAvailabilityModelForWeek(rw.getWeekOfYear());
				if ( am == null ) {
				
					w.addElement("empty").setText("true");
					continue;
				}
				
				double availability = am.getAverageAvailablility();
				w.addElement("availability").setText(percentFormat.format(availability));
				w.addElement("scheduled-downtime").setText(percentFormat.format(am.getScheduledDowntime()));
				w.addElement("unscheduled-downtime").setText(percentFormat.format(am.getUnscheduledDowntime()));
				
				double warning = am.getHostGroup().getWarning();
				double critical = am.getHostGroup().getCritical();
				
				if ( availability <= warning && availability > critical )
					w.addAttribute("color", "#" + ODWReportBC.HEX_WARNING);
				else if ( availability <= critical )
					w.addAttribute("color", "red");
				else
					w.addAttribute("color", "black");
			}
		}

		return exceptionDetail;
	}

	/**
	 * 
	 */
	private List<ReportWeek> listReportWeeks() {
		
		Date dataStartDate = rc.getDataStartDate();
		Date dataEndDate = rc.getDataEndDate();

		// get first week and add it to the list
		Calendar cal = Calendar.getInstance();
		cal.setTimeInMillis(dataStartDate.getTime());
		cal.setFirstDayOfWeek(Calendar.MONDAY);
		cal.setMinimalDaysInFirstWeek(4);

		int weekOfYear = cal.get(Calendar.WEEK_OF_YEAR);
		
		ReportWeek week = 
			new ReportWeek(String.valueOf(cal.get(Calendar.YEAR)), String.valueOf(weekOfYear), dataStartDate, dataEndDate);
		List<ReportWeek> weeks = new ArrayList<ReportWeek>();
		weeks.add(week);

		//now do the rest of the weeks
		while (!week.isLastWeekInReport()) {
			Long millis = week.getWeekStartDate().getTime()+(1000 * 60 * 60 * 24 *7);
			
			Calendar nextCal = Calendar.getInstance();
			nextCal.setTimeInMillis(millis);
			int year = nextCal.get(Calendar.YEAR);
			if(weekOfYear > nextCal.get(Calendar.WEEK_OF_YEAR))
				year++;
				
					
			String y = String.valueOf(year);
			weekOfYear = nextCal.get(Calendar.WEEK_OF_YEAR);
			

			week = new ReportWeek(y, String.valueOf(weekOfYear), dataStartDate, dataEndDate);
			weeks.add(week);
		}

		return weeks;

	}
	
	/**
	 * 
	 */
	public void runQuery() throws ReportException {
		
		l = dao.getWeeklyAvailabilityForHostGroups(configDao, rc);
	}
}
